@@ -2,8 +2,6 @@
 
 set -eu -o pipefail
 
-pip install twine sphinx
-
 changelog=$(cat HISTORY.rst)
 
 regex='
@@ -36,6 +34,7 @@ if [ -n "$(git status --porcelain)" ]; then
 fi
 
 perl -pi -e "s/(?<=__version__ = \").+?(?=\")/$version/gsm" maxminddb/__init__.py
+perl -pi -e "s/(?<=^version = \").+?(?=\")/$version/gsm" pyproject.toml
 
 echo $"Test results:"
 tox
@@ -62,8 +61,3 @@ git push
 gh release create --target "$(git branch --show-current)" -t "$version" -n "$notes" "$tag"
 
 git push --tags
-
-rm -fr build dist
-python -m sphinx -M html ./docs ./build/sphinx -W
-python setup.py sdist
-twine upload dist/*
