@@ -1,16 +1,26 @@
+#define PY_SSIZE_T_CLEAN
 #include <Python.h>
+#ifdef MS_WINDOWS
+#include <winsock2.h>
+#include <ws2tcpip.h>
+#else
 #include <arpa/inet.h>
-#include <maxminddb.h>
 #include <netinet/in.h>
-#include <structmember.h>
 #include <sys/socket.h>
+#include <unistd.h>
+#endif
+#include <maxminddb.h>
+#include <stdbool.h>
+#include <structmember.h>
 
 #define __STDC_FORMAT_MACROS
 #include <inttypes.h>
 
 static PyTypeObject Reader_Type;
+static PyTypeObject ReaderIter_Type;
 static PyTypeObject Metadata_Type;
 static PyObject *MaxMindDB_error;
+static PyObject *ipaddress_ip_network;
 
 // clang-format off
 typedef struct {
@@ -19,6 +29,22 @@ typedef struct {
     PyObject *closed;
 } Reader_obj;
 
+typedef struct record record;
+struct record{
+    char ip_packed[16];
+    int depth;
+    uint64_t record;
+    uint8_t type;
+    MMDB_entry_s entry;
+    struct record *next;
+};
+
+typedef struct {
+    PyObject_HEAD /* no semicolon */
+    Reader_obj *reader;
+    struct record *next;
+} ReaderIter_obj;
+
 typedef struct {
     PyObject_HEAD /* no semicolon */
     PyObject *binary_format_major_version;
@@ -33,6 +59,7 @@ typedef struct {
 } Metadata_obj;
 // clang-format on
 
+static bool can_read(const char *path);
 static int get_record(PyObject *self, PyObject *args, PyObject **record);
 static bool format_sockaddr(struct sockaddr *addr, char *dst);
 static PyObject *from_entry_data_list(MMDB_entry_data_list_s **entry_data_list);
@@ -77,15 +104,14 @@ static int Reader_init(PyObject *self, PyObject *args, PyObject *kwds) {
         return -1;
     }
 
-    if (0 != access(filename, R_OK)) {
-
+    if (!can_read(filename)) {
         PyErr_SetFromErrnoWithFilenameObject(PyExc_OSError, filepath);
         Py_XDECREF(filepath);
         return -1;
     }
 
     MMDB_s *mmdb = (MMDB_s *)malloc(sizeof(MMDB_s));
-    if (NULL == mmdb) {
+    if (mmdb == NULL) {
         Py_XDECREF(filepath);
         PyErr_NoMemory();
         return -1;
@@ -99,18 +125,20 @@ static int Reader_init(PyObject *self, PyObject *args, PyObject *kwds) {
         return -1;
     }
 
-    uint16_t status = MMDB_open(filename, MMDB_MODE_MMAP, mmdb);
-    Py_XDECREF(filepath);
+    int const status = MMDB_open(filename, MMDB_MODE_MMAP, mmdb);
 
-    if (MMDB_SUCCESS != status) {
+    if (status != MMDB_SUCCESS) {
         free(mmdb);
         PyErr_Format(MaxMindDB_error,
                      "Error opening database file (%s). Is this a valid "
                      "MaxMind DB file?",
                      filename);
+        Py_XDECREF(filepath);
         return -1;
     }
 
+    Py_XDECREF(filepath);
+
     mmdb_obj->mmdb = mmdb;
     mmdb_obj->closed = Py_False;
     return 0;
@@ -139,7 +167,7 @@ static PyObject *Reader_get_with_prefix_len(PyObject *self, PyObject *args) {
 
 static int get_record(PyObject *self, PyObject *args, PyObject **record) {
     MMDB_s *mmdb = ((Reader_obj *)self)->mmdb;
-    if (NULL == mmdb) {
+    if (mmdb == NULL) {
         PyErr_SetString(PyExc_ValueError,
                         "Attempt to read from a closed MaxMind DB.");
         return -1;
@@ -160,7 +188,7 @@ static int get_record(PyObject *self, PyObject *args, PyObject **record) {
     MMDB_lookup_result_s result =
         MMDB_lookup_sockaddr(mmdb, ip_address, &mmdb_error);
 
-    if (MMDB_SUCCESS != mmdb_error) {
+    if (mmdb_error != MMDB_SUCCESS) {
         PyObject *exception;
         if (MMDB_IPV6_LOOKUP_IN_IPV4_DATABASE_ERROR == mmdb_error) {
             exception = PyExc_ValueError;
@@ -192,7 +220,7 @@ static int get_record(PyObject *self, PyObject *args, PyObject **record) {
 
     MMDB_entry_data_list_s *entry_data_list = NULL;
     int status = MMDB_get_entry_data_list(&result.entry, &entry_data_list);
-    if (MMDB_SUCCESS != status) {
+    if (status != MMDB_SUCCESS) {
         char ipstr[INET6_ADDRSTRLEN] = {0};
         if (format_sockaddr(ip_address, ipstr)) {
             PyErr_Format(MaxMindDB_error,
@@ -276,14 +304,14 @@ static int ip_converter(PyObject *obj, struct sockaddr_storage *ip_address) {
         case 16: {
             ip_address->ss_family = AF_INET6;
             struct sockaddr_in6 *sin = (struct sockaddr_in6 *)ip_address;
-            memcpy(sin->sin6_addr.s6_addr, bytes, len);
+            memcpy(sin->sin6_addr.s6_addr, bytes, (size_t)len);
             Py_DECREF(packed);
             return 1;
         }
         case 4: {
             ip_address->ss_family = AF_INET;
             struct sockaddr_in *sin = (struct sockaddr_in *)ip_address;
-            memcpy(&(sin->sin_addr.s_addr), bytes, len);
+            memcpy(&(sin->sin_addr.s_addr), bytes, (size_t)len);
             Py_DECREF(packed);
             return 1;
         }
@@ -316,7 +344,7 @@ static bool format_sockaddr(struct sockaddr *sa, char *dst) {
 static PyObject *Reader_metadata(PyObject *self, PyObject *UNUSED(args)) {
     Reader_obj *mmdb_obj = (Reader_obj *)self;
 
-    if (NULL == mmdb_obj->mmdb) {
+    if (mmdb_obj->mmdb == NULL) {
         PyErr_SetString(PyExc_IOError,
                         "Attempt to read from a closed MaxMind DB.");
         return NULL;
@@ -328,13 +356,13 @@ static PyObject *Reader_metadata(PyObject *self, PyObject *UNUSED(args)) {
 
     PyObject *metadata_dict = from_entry_data_list(&entry_data_list);
     MMDB_free_entry_data_list(original_entry_data_list);
-    if (NULL == metadata_dict || !PyDict_Check(metadata_dict)) {
+    if (metadata_dict == NULL || !PyDict_Check(metadata_dict)) {
         PyErr_SetString(MaxMindDB_error, "Error decoding metadata.");
         return NULL;
     }
 
     PyObject *args = PyTuple_New(0);
-    if (NULL == args) {
+    if (args == NULL) {
         Py_DECREF(metadata_dict);
         return NULL;
     }
@@ -349,7 +377,7 @@ static PyObject *Reader_metadata(PyObject *self, PyObject *UNUSED(args)) {
 static PyObject *Reader_close(PyObject *self, PyObject *UNUSED(args)) {
     Reader_obj *mmdb_obj = (Reader_obj *)self;
 
-    if (NULL != mmdb_obj->mmdb) {
+    if (mmdb_obj->mmdb != NULL) {
         MMDB_close(mmdb_obj->mmdb);
         free(mmdb_obj->mmdb);
         mmdb_obj->mmdb = NULL;
@@ -380,13 +408,210 @@ static PyObject *Reader__exit__(PyObject *self, PyObject *UNUSED(args)) {
 
 static void Reader_dealloc(PyObject *self) {
     Reader_obj *obj = (Reader_obj *)self;
-    if (NULL != obj->mmdb) {
+    if (obj->mmdb != NULL) {
         Reader_close(self, NULL);
     }
 
     PyObject_Del(self);
 }
 
+static PyObject *Reader_iter(PyObject *obj) {
+    Reader_obj *reader = (Reader_obj *)obj;
+    if (reader->closed == Py_True) {
+        PyErr_SetString(PyExc_ValueError,
+                        "Attempt to iterate over a closed MaxMind DB.");
+        return NULL;
+    }
+
+    ReaderIter_obj *ri = PyObject_New(ReaderIter_obj, &ReaderIter_Type);
+    if (ri == NULL) {
+        return NULL;
+    }
+
+    ri->reader = reader;
+    Py_INCREF(reader);
+
+    // Currently, we are always starting from the 0 node with the 0 IP
+    ri->next = calloc(1, sizeof(record));
+    if (ri->next == NULL) {
+        // ReaderIter_dealloc will decrement the reference count on reader
+        Py_DECREF(ri);
+        PyErr_NoMemory();
+        return NULL;
+    }
+
+    return (PyObject *)ri;
+}
+
+static bool is_ipv6(char ip[16]) {
+    char z = 0;
+    for (int i = 0; i < 12; i++) {
+        z |= ip[i];
+    }
+    return z;
+}
+
+static PyObject *ReaderIter_next(PyObject *self) {
+    ReaderIter_obj *ri = (ReaderIter_obj *)self;
+    if (ri->reader->closed == Py_True) {
+        PyErr_SetString(PyExc_ValueError,
+                        "Attempt to iterate over a closed MaxMind DB.");
+        return NULL;
+    }
+
+    while (ri->next != NULL) {
+        record *cur = ri->next;
+        ri->next = cur->next;
+
+        switch (cur->type) {
+            case MMDB_RECORD_TYPE_INVALID:
+                PyErr_SetString(MaxMindDB_error,
+                                "Invalid record when reading node");
+                free(cur);
+                return NULL;
+            case MMDB_RECORD_TYPE_SEARCH_NODE: {
+                if (cur->record ==
+                        ri->reader->mmdb->ipv4_start_node.node_value &&
+                    is_ipv6(cur->ip_packed)) {
+                    // These are aliased networks. Skip them.
+                    break;
+                }
+                MMDB_search_node_s node;
+                int status = MMDB_read_node(
+                    ri->reader->mmdb, (uint32_t)cur->record, &node);
+                if (status != MMDB_SUCCESS) {
+                    const char *error = MMDB_strerror(status);
+                    PyErr_Format(
+                        MaxMindDB_error, "Error reading node: %s", error);
+                    free(cur);
+                    return NULL;
+                }
+                struct record *left = calloc(1, sizeof(record));
+                if (left == NULL) {
+                    free(cur);
+                    PyErr_NoMemory();
+                    return NULL;
+                }
+                memcpy(
+                    left->ip_packed, cur->ip_packed, sizeof(left->ip_packed));
+                left->depth = cur->depth + 1;
+                left->record = node.left_record;
+                left->type = node.left_record_type;
+                left->entry = node.left_record_entry;
+
+                struct record *right = left->next = calloc(1, sizeof(record));
+                if (right == NULL) {
+                    free(cur);
+                    free(left);
+                    PyErr_NoMemory();
+                    return NULL;
+                }
+                memcpy(
+                    right->ip_packed, cur->ip_packed, sizeof(right->ip_packed));
+                right->ip_packed[cur->depth / 8] |= 1 << (7 - cur->depth % 8);
+                right->depth = cur->depth + 1;
+                right->record = node.right_record;
+                right->type = node.right_record_type;
+                right->entry = node.right_record_entry;
+                right->next = ri->next;
+
+                ri->next = left;
+                break;
+            }
+            case MMDB_RECORD_TYPE_EMPTY:
+                break;
+            case MMDB_RECORD_TYPE_DATA: {
+                MMDB_entry_data_list_s *entry_data_list = NULL;
+                int status =
+                    MMDB_get_entry_data_list(&cur->entry, &entry_data_list);
+                if (status != MMDB_SUCCESS) {
+                    PyErr_Format(
+                        MaxMindDB_error,
+                        "Error looking up data while iterating over tree: %s",
+                        MMDB_strerror(status));
+                    MMDB_free_entry_data_list(entry_data_list);
+                    free(cur);
+                    return NULL;
+                }
+
+                MMDB_entry_data_list_s *original_entry_data_list =
+                    entry_data_list;
+                PyObject *record = from_entry_data_list(&entry_data_list);
+                MMDB_free_entry_data_list(original_entry_data_list);
+                if (record == NULL) {
+                    free(cur);
+                    return NULL;
+                }
+
+                int ip_start = 0;
+                int ip_length = 4;
+                if (ri->reader->mmdb->depth == 128) {
+                    if (is_ipv6(cur->ip_packed)) {
+                        // IPv6 address
+                        ip_length = 16;
+                    } else {
+                        // IPv4 address in IPv6 tree
+                        ip_start = 12;
+                    }
+                }
+                PyObject *network_tuple =
+                    Py_BuildValue("(y#i)",
+                                  &(cur->ip_packed[ip_start]),
+                                  ip_length,
+                                  cur->depth - ip_start * 8);
+                if (network_tuple == NULL) {
+                    Py_DECREF(record);
+                    free(cur);
+                    return NULL;
+                }
+                PyObject *args = PyTuple_Pack(1, network_tuple);
+                Py_DECREF(network_tuple);
+                if (args == NULL) {
+                    Py_DECREF(record);
+                    free(cur);
+                    return NULL;
+                }
+                PyObject *network =
+                    PyObject_CallObject(ipaddress_ip_network, args);
+                Py_DECREF(args);
+                if (network == NULL) {
+                    Py_DECREF(record);
+                    free(cur);
+                    return NULL;
+                }
+
+                PyObject *rv = PyTuple_Pack(2, network, record);
+                Py_DECREF(network);
+                Py_DECREF(record);
+
+                free(cur);
+                return rv;
+            }
+            default:
+                PyErr_Format(
+                    MaxMindDB_error, "Unknown record type: %u", cur->type);
+                free(cur);
+                return NULL;
+        }
+        free(cur);
+    }
+    return NULL;
+}
+
+static void ReaderIter_dealloc(PyObject *self) {
+    ReaderIter_obj *ri = (ReaderIter_obj *)self;
+
+    Py_DECREF(ri->reader);
+
+    struct record *next = ri->next;
+    while (next != NULL) {
+        struct record *cur = next;
+        next = cur->next;
+        free(cur);
+    }
+    PyObject_Del(self);
+}
+
 static int Metadata_init(PyObject *self, PyObject *args, PyObject *kwds) {
 
     PyObject *binary_format_major_version, *binary_format_minor_version,
@@ -461,7 +686,7 @@ static void Metadata_dealloc(PyObject *self) {
 
 static PyObject *
 from_entry_data_list(MMDB_entry_data_list_s **entry_data_list) {
-    if (NULL == entry_data_list || NULL == *entry_data_list) {
+    if (entry_data_list == NULL || *entry_data_list == NULL) {
         PyErr_SetString(MaxMindDB_error,
                         "Error while looking up data. Your database may be "
                         "corrupt or you have found a bug in libmaxminddb.");
@@ -511,14 +736,14 @@ from_entry_data_list(MMDB_entry_data_list_s **entry_data_list) {
 
 static PyObject *from_map(MMDB_entry_data_list_s **entry_data_list) {
     PyObject *py_obj = PyDict_New();
-    if (NULL == py_obj) {
+    if (py_obj == NULL) {
         PyErr_NoMemory();
         return NULL;
     }
 
     const uint32_t map_size = (*entry_data_list)->entry_data.data_size;
 
-    uint i;
+    uint32_t i;
     // entry_data_list cannot start out NULL (see from_entry_data_list). We
     // check it in the loop because it may become NULL.
     // coverity[check_after_deref]
@@ -526,7 +751,7 @@ static PyObject *from_map(MMDB_entry_data_list_s **entry_data_list) {
         *entry_data_list = (*entry_data_list)->next;
 
         PyObject *key = PyUnicode_FromStringAndSize(
-            (char *)(*entry_data_list)->entry_data.utf8_string,
+            (*entry_data_list)->entry_data.utf8_string,
             (*entry_data_list)->entry_data.data_size);
         if (!key) {
             // PyUnicode_FromStringAndSize will set an appropriate exception
@@ -537,7 +762,7 @@ static PyObject *from_map(MMDB_entry_data_list_s **entry_data_list) {
         *entry_data_list = (*entry_data_list)->next;
 
         PyObject *value = from_entry_data_list(entry_data_list);
-        if (NULL == value) {
+        if (value == NULL) {
             Py_DECREF(key);
             Py_DECREF(py_obj);
             return NULL;
@@ -554,19 +779,19 @@ static PyObject *from_array(MMDB_entry_data_list_s **entry_data_list) {
     const uint32_t size = (*entry_data_list)->entry_data.data_size;
 
     PyObject *py_obj = PyList_New(size);
-    if (NULL == py_obj) {
+    if (py_obj == NULL) {
         PyErr_NoMemory();
         return NULL;
     }
 
-    uint i;
+    uint32_t i;
     // entry_data_list cannot start out NULL (see from_entry_data_list). We
     // check it in the loop because it may become NULL.
     // coverity[check_after_deref]
     for (i = 0; i < size && entry_data_list; i++) {
         *entry_data_list = (*entry_data_list)->next;
         PyObject *value = from_entry_data_list(entry_data_list);
-        if (NULL == value) {
+        if (value == NULL) {
             Py_DECREF(py_obj);
             return NULL;
         }
@@ -594,7 +819,7 @@ static PyObject *from_uint128(const MMDB_entry_data_list_s *entry_data_list) {
 #endif
 
     char *num_str = malloc(33);
-    if (NULL == num_str) {
+    if (num_str == NULL) {
         PyErr_NoMemory();
         return NULL;
     }
@@ -607,6 +832,16 @@ static PyObject *from_uint128(const MMDB_entry_data_list_s *entry_data_list) {
     return py_obj;
 }
 
+static bool can_read(const char *path) {
+#ifdef MS_WINDOWS
+    int rv = _access(path, 04);
+#else
+    int rv = access(path, R_OK);
+#endif
+
+    return rv == 0;
+}
+
 static PyMethodDef Reader_methods[] = {
     {"get",
      Reader_get,
@@ -642,6 +877,7 @@ static PyTypeObject Reader_Type = {
     .tp_dealloc = Reader_dealloc,
     .tp_doc = "Reader object",
     .tp_flags = Py_TPFLAGS_DEFAULT,
+    .tp_iter = Reader_iter,
     .tp_methods = Reader_methods,
     .tp_members = Reader_members,
     .tp_name = "Reader",
@@ -649,6 +885,22 @@ static PyTypeObject Reader_Type = {
 };
 // clang-format on
 
+static PyMethodDef ReaderIter_methods[] = {{NULL, NULL, 0, NULL}};
+
+// clang-format off
+static PyTypeObject ReaderIter_Type = {
+    PyVarObject_HEAD_INIT(NULL, 0)
+    .tp_basicsize = sizeof(ReaderIter_obj),
+    .tp_dealloc = ReaderIter_dealloc,
+    .tp_doc = "Iterator for Reader object",
+    .tp_flags = Py_TPFLAGS_DEFAULT,
+    .tp_iter = PyObject_SelfIter,
+    .tp_iternext = ReaderIter_next,
+    .tp_methods = ReaderIter_methods,
+    .tp_name = "ReaderIter",
+};
+// clang-format on
+
 static PyMethodDef Metadata_methods[] = {{NULL, NULL, 0, NULL}};
 
 static PyMemberDef Metadata_members[] = {
@@ -737,6 +989,7 @@ PyMODINIT_FUNC PyInit_extension(void) {
     if (PyType_Ready(&Metadata_Type)) {
         return NULL;
     }
+    Py_INCREF(&Metadata_Type);
     PyModule_AddObject(m, "Metadata", (PyObject *)&Metadata_Type);
 
     PyObject *error_mod = PyImport_ImportModule("maxminddb.errors");
@@ -750,9 +1003,21 @@ PyMODINIT_FUNC PyInit_extension(void) {
     if (MaxMindDB_error == NULL) {
         return NULL;
     }
-
     Py_INCREF(MaxMindDB_error);
 
+    PyObject *ipaddress_mod = PyImport_ImportModule("ipaddress");
+    if (ipaddress_mod == NULL) {
+        return NULL;
+    }
+
+    ipaddress_ip_network = PyObject_GetAttrString(ipaddress_mod, "ip_network");
+    Py_DECREF(ipaddress_mod);
+
+    if (ipaddress_ip_network == NULL) {
+        return NULL;
+    }
+    Py_INCREF(ipaddress_ip_network);
+
     /* We primarily add it to the module for backwards compatibility */
     PyModule_AddObject(m, "InvalidDatabaseError", MaxMindDB_error);
 
